@@ -1,16 +1,16 @@
 #!/usr/bin/perl
 #
-# $Id: tcp-scan-multi.pl 34 2015-02-20 06:16:17Z gomor $
+# $Id: tcp-scan-multi.pl 35 2015-02-20 06:38:19Z gomor $
 #
 use strict;
 use warnings;
 
 use Getopt::Std;
 my %opts;
-getopts('t:p:r:n:TDF6', \%opts);
+getopts('t:p:r:n:TDF6d:s:', \%opts);
 
 if (!$opts{t}) {
-   die("Usage: $0 -t targets -p ports -r rate -n ntimes [-D] [-F] [-6]\n");
+   die("Usage: $0 -t targets -p ports -r rate -n ntimes [-D] [-F] [-6] [-d device] [-s source IP]\n");
 }
 
 use Net::Netmask;
@@ -43,22 +43,38 @@ else {
    push @targets, $target;
 }
 
-my $nr;
-(my $nr_target = $targets[0]) =~ s/\/\d+$//;
-if ($opts{6}) {
-   $nr = Net::Routing->new(target => $nr_target, family => NR_FAMILY_INET6());
+my $interface;
+
+if (! $opts{d}) {
+   my $nr;
+   (my $nr_target = $targets[0]) =~ s/\/\d+$//;
+   if ($opts{6}) {
+      $nr = Net::Routing->new(target => $nr_target, family => NR_FAMILY_INET6());
+   }
+   else {
+      $nr = Net::Routing->new(target => $nr_target, family => NR_FAMILY_INET4());
+   }
+   my $routes = $nr->get or die("get: $Net::Routing::Error");
+   $interface = $routes->[0]->{interface};
+   if (@$routes != 1) {
+      eval("use Data::Dumper;");
+      die("Routing not found in: ".Dumper($routes)."\n");
+   }
 }
 else {
-   $nr = Net::Routing->new(target => $nr_target, family => NR_FAMILY_INET4());
+   $interface = $opts{d};
 }
 
-my $routes = $nr->get;
-my $dev;
-if (@$routes == 1) {
-   $dev = Net::Frame::Device->new(dev => $routes->[0]->{interface});
+my $dev = Net::Frame::Device->new(dev => $interface);
+my $ip;
+my $ip6;
+if ($opts{6}) {
+   $ip6 = $opts{s} || $dev->ip6;
+   print("Using source IPv6 address: $ip6\n");
 }
 else {
-   die("Multiple routes match\n");
+   $ip = $opts{s} || $dev->ip;
+   print("Using source IPv4 address: $ip\n");
 }
 
 my @full = (1..65535);
@@ -181,7 +197,7 @@ my $d = Net::Frame::Dump::Online2->new(
 );
 if ($opts{6}) {
    $d->filter(
-      'ip6 and src host '.$target.' and dst host '.$dev->ip6,
+      'ip6 and src host '.$target.' and dst host '.$ip6,
    );
 }
 else {
@@ -211,7 +227,7 @@ defined(my $pid = fork()) or die("Can't fork() [$!]\n");
 if (! $pid) { # Son
    #print STDERR "DEBUG: run send()\n";
    my $r = Net::Write::Fast::l4_send_tcp_syn_multi(
-      $opts{6} ? $dev->ip6 : $dev->ip,
+      $opts{6} ? $ip6 : $ip,
       \@targets,
       \@send,
       $pps,
